@@ -36,7 +36,9 @@ __author__ = "Karsten.Hilbert@gmx.net"
 __license__ = "GPL v2 or later"
 
 # standard library
-import sys, string, os.path, fileinput, os, time, getpass, glob, re as regex, tempfile, logging
+import sys, string, os.path, fileinput, os, time, getpass, glob, re as regex, tempfile
+import io
+import logging
 
 
 # adjust Python path
@@ -711,6 +713,16 @@ class database:
 		except:
 			_log.exception('PostgreSQL version < 9.3 does not support <ignore_checksum_failure>')
 		curs.close()
+
+		# we want to track commit timestamps if available
+		# remove exception handler when 9.5 is default
+		curs = self.conn.cursor()
+		try:
+			curs.execute("alter database %s set track_commit_timestamp to on" % self.name)
+		except:
+			_log.exception('PostgreSQL version < 9.5 does not support <track_commit_timestamp> OR <track_commit_timestamp> cannot be set at runtime')
+		curs.close()
+
 		self.conn.commit()
 
 		curs = self.conn.cursor()
@@ -880,9 +892,26 @@ class database:
 		)
 		target_conn.cookie = 'check_data_plausibility: target'
 
+		all_tests_successful = True
+
 		for idx in range(no_of_queries):
-			tag, old_query = plausibility_queries[idx*2].split('::::')
+			check_def = plausibility_queries[idx*2]
+			if check_def.startswith('--'):
+				_log.debug('skipped: %s', check_def)
+				continue
+
+			tag = u'?'
+			old_query = u'?'
+			try:
+				tag, old_query = check_def.split('::::')
+			except:
+				_log.exception('error in plausibility check, aborting')
+				_log.error('check definition: %s', check_def)
+				print_msg("    ... failed (check definition error)")
+				all_tests_successful = False
+				continue
 			new_query = plausibility_queries[(idx*2) + 1]
+
 			try:
 				rows, idx = gmPG2.run_ro_queries (
 					link_obj = template_conn,
@@ -891,8 +920,11 @@ class database:
 				old_val = rows[0][0]
 			except:
 				_log.exception('error in plausibility check [%s] (old), aborting' % tag)
+				_log.error('SQL: %s', old_query)
 				print_msg("    ... failed (SQL error)")
-				return False
+				all_tests_successful = False
+				continue
+
 			try:
 				rows, idx = gmPG2.run_ro_queries (
 					link_obj = target_conn,
@@ -901,20 +933,25 @@ class database:
 				new_val = rows[0][0]
 			except:
 				_log.exception('error in plausibility check [%s] (new), aborting' % tag)
+				_log.error('SQL: %s', new_query)
 				print_msg("    ... failed (SQL error)")
-				return False
+				all_tests_successful = False
+				continue
 
 			if new_val != old_val:
 				_log.error('plausibility check [%s] failed, expected [%s], found [%s]' % (tag, old_val, new_val))
-				print_msg("    ... failed (check [%s])" % tag)
-				return False
+				_log.error('SQL (old DB): %s', old_query)
+				_log.error('SQL (new DB): %s', new_query)
+				print_msg("    ... failed (data error, check [%s])" % tag)
+				all_tests_successful = False
+				continue
 
 			_log.info('plausibility check [%s] succeeded' % tag)
 
 		template_conn.close()
 		target_conn.close()
 
-		return True
+		return all_tests_successful
 	#--------------------------------------------------------------
 	def check_holy_auth_line(self):
 
@@ -941,9 +978,8 @@ class database:
 		_log.info('hba file: %s', hba_file)
 
 		try:
-			f = open(hba_file, 'r')
-			f.close()
-		except StandardError:
+			f = io.open(hba_file, mode = 'rt').close()
+		except Exception:
 			_log.exception('cannot check pg_hba.conf for authentication information - not readable')
 			return
 
@@ -1058,22 +1094,36 @@ class database:
 
 	#--------------------------------------------------------------
 	def reindex_all(self):
-		print_msg("==> reindexing target database ...")
-		_log.info('REINDEXing cloned target database so upgrade does not fail because of a broken index')
+
+		print_msg("==> reindexing target database (can take a while) ...")
+		_log.info('REINDEXing cloned target database so upgrade does not fail in case of a broken index')
+		_log.info('this may potentially take "quite a long time" depending on how much data there is in the database')
+		_log.info('you may want to monitor the PostgreSQL log for signs of progress')
+
 		old_iso = self.conn.isolation_level
 		self.conn.set_isolation_level(0)
 		curs = self.conn.cursor()
-		cmd = 'REINDEX DATABASE %s' % self.name
+		cmd = 'REINDEX (VERBOSE) DATABASE %s' % self.name
 		try:
 			curs.execute(cmd)
 		except:
 			_log.exception(">>>[%s]<<< failed" % cmd)
 			curs.close()
-			self.conn.set_isolation_level(old_iso)
-			return False
+			# re-attempt w/o VERBOSE
+			_log.info('attempting REINDEXing without VERBOSE')
+			curs = self.conn.cursor()
+			cmd = 'REINDEX DATABASE %s' % self.name
+			try:
+				curs.execute(cmd)
+			except:
+				_log.exception(">>>[%s]<<< failed" % cmd)
+				curs.close()
+				self.conn.set_isolation_level(old_iso)
+				return False
 		curs.close()
 		self.conn.set_isolation_level(old_iso)
 		return True
+
 	#--------------------------------------------------------------
 	def transfer_users(self):
 		print_msg("==> transferring users ...")
@@ -1103,6 +1153,7 @@ class database:
 		_log.error('error transferring user from [%s] to [%s]' % (self.template_db, self.name))
 		print_msg("    ... failed")
 		return False
+
 	#--------------------------------------------------------------
 	def bootstrap_auditing(self):
 		print_msg("==> setting up auditing ...")
@@ -1139,10 +1190,10 @@ class database:
 			return None
 		# write schema to file
 		tmpfile = os.path.join(tempfile.gettempdir(), 'audit-trail-schema.sql')
-		file = open(tmpfile, 'wb')
+		f = io.open(tmpfile, mode = 'wt', encoding = 'utf8')
 		for line in audit_schema:
-			file.write("%s;\n" % line)
-		file.close()
+			f.write(u'%s;\n' % line)
+		f.close()
 
 		# import auditing schema
 		psql = gmPsql.Psql(self.conn)
@@ -1155,7 +1206,7 @@ class database:
 
 		try:
 			os.remove(tmpfile)
-		except StandardError:
+		except Exception:
 			_log.exception('cannot remove audit trail schema file [%s]' % tmpfile)
 		return True
 
@@ -1669,7 +1720,7 @@ if __name__ == "__main__":
 
 	try:
 		main()
-	except StandardError:
+	except Exception:
 		for c in conn_ref_count:
 			if c.closed == 0:
 				print 'closing open connection from:', c.cookie
